@@ -175,9 +175,19 @@ class SimpleTree:
                 options.min_year=min_year
                 print 'Using min year {0} for experiment {1}'.format(str(min_year),experiment)
 
+        #Start the retrieval workers:
+        if not ('serial' in dir(options) and options.serial):
+            processes=dict()
+            for data_node in data_node_list:
+                processes[data_node]=multiprocessing.Process(target=worker_retrieve, args=(queues[data_node], queues['end']))
+                processes[data_node].start()
+
+        #Find the data that needs to be recovered:
         self.define_database(options)
         self.nc_Database.retrieve_database(options,output,queues,getattr(retrieval_utils,retrieval_function))
         self.close_database()
+
+        #Launch the retrieval/monitoring:
         launch_download_and_remote_retrieve(output,data_node_list,queues,retrieval_function,options)
         return
 
@@ -322,6 +332,7 @@ def worker_retrieve(input, output):
     for tuple in iter(input.get, 'STOP'):
         result = tuple[0](tuple[1],tuple[2])
         output.put(result)
+    output.put('STOP')
     return
 
 def launch_download_and_remote_retrieve(output,data_node_list,queues,retrieval_function,options):
@@ -329,35 +340,32 @@ def launch_download_and_remote_retrieve(output,data_node_list,queues,retrieval_f
     #print datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
     start_time = datetime.datetime.now()
     print('Retrieving from data nodes:')
-    data_node_list_not_empty=[ data_node for data_node in data_node_list
-                                    if queues[data_node].qsize()>0]
     queues_size=dict()
-    for data_node in data_node_list_not_empty:
+    for data_node in data_node_list:
         queues_size[data_node]=queues[data_node].qsize()
     string_to_print=['0'.zfill(len(str(queues_size[data_node])))+'/'+str(queues_size[data_node])+' paths from "'+data_node+'"' for
-                        data_node in data_node_list_not_empty]
+                        data_node in data_node_list]
     print ' | '.join(string_to_print)
     print 'Progress: '
 
     if 'serial' in dir(options) and options.serial:
-        for data_node in data_node_list_not_empty:
-            num_files=queues[data_node].qsize()
+        for data_node in data_node_list:
+            num_files=queues_size[data_node]
             queues[data_node].put('STOP')
             worker_retrieve(queues[data_node], queues['end'])
-            for i in range(num_files):
-                progress_report(retrieval_function,output,queues,queues_size,data_node_list_not_empty,start_time)
+            for tuple in iter(queues['end'].get, 'STOP'):
+                progress_report(retrieval_function,output,tuple,queues,queues_size,data_node_list,start_time)
         
     else:
         num_files=0
-        processes=dict()
-        for data_node in data_node_list_not_empty:
-            num_files+=queues[data_node].qsize()
+        for data_node in data_node_list:
             queues[data_node].put('STOP')
-            processes[data_node]=multiprocessing.Process(target=worker_retrieve, args=(queues[data_node], queues['end']))
-            processes[data_node].start()
 
-        for i in range(num_files):
-            progress_report(retrieval_function,output,queues,queues_size,data_node_list_not_empty,start_time)
+        for data_node in data_node_list:
+            for tuple in iter(queues['end'].get, 'STOP'):
+                progress_report(retrieval_function,output,tuple,queues,queues_size,data_node_list,start_time)
+            #print data_node
+            #print queues['end'].qsize()
 
     if retrieval_function=='retrieve_path_data':
         output.close()
@@ -367,17 +375,19 @@ def launch_download_and_remote_retrieve(output,data_node_list,queues,retrieval_f
     #print datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
     return
 
-def progress_report(retrieval_function,output,queues,queues_size,data_node_list_not_empty,start_time):
+def progress_report(retrieval_function,output,tuple,queues,queues_size,data_node_list,start_time):
     if retrieval_function=='retrieve_path':
-        print '\t', queues['end'].get()
+        #print '\t', queues['end'].get()
+        print '\t', tuple
         elapsed_time = datetime.datetime.now() - start_time
         print str(elapsed_time)
     elif retrieval_function=='retrieve_path_data':
-        netcdf_utils.assign_tree(output,*queues['end'].get())
+        #netcdf_utils.assign_tree(output,*queues['end'].get())
+        netcdf_utils.assign_tree(output,*tuple)
         output.sync()
         string_to_print=[str(queues_size[data_node]-queues[data_node].qsize()).zfill(len(str(queues_size[data_node])))+
                          '/'+str(queues_size[data_node]) for
-                            data_node in data_node_list_not_empty]
+                            data_node in data_node_list]
         elapsed_time = datetime.datetime.now() - start_time
         print str(elapsed_time)+', '+' | '.join(string_to_print)+'\r',
     return
